﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Net;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Windows.Media.Imaging;
using UIFramework.Controls;
using System.Diagnostics;
using Microsoft.FaceSdk.ImageHelper;
using Microsoft.Devices;
using Microsoft.Xna.Framework;
using System.IO;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Phone.BackgroundAudio;

namespace FaceParty
{
    public partial class EditPage : PhoneApplicationPage
    {
        public bool IsProcessing
        {
            get { return (this.faceDetectingAnimation.IsFaceDetecting||this.faceDetectingAnimation.IsAnimationRunning); }
        }
        private WriteableBitmap GetRawBitmap()
        {
            if (rawBitmap == null)
            {
                rawBitmap = ImageConverter.SdkToSystem(rawImage);
            }

            return rawBitmap;
        }

        private void Reset()
        {
            // clear all face effect and load original face picture;
            this.imgViewer.Source = GetRawBitmap();

            // show all mask controls;
            this.maskCanvas.Visibility = System.Windows.Visibility.Visible;

            List<FaceWrapper> nonShownFaces = new List<FaceWrapper>();
            if (this.currentFaces != null)
            {
                foreach (var f in this.currentFaces) nonShownFaces.Add(f);
            }

            foreach (var element in this.maskCanvas.Children)
            {
                var mask = element as MaskControl;
                if (mask != null)
                {
                    mask.DataSource = null;
                    mask.ArrowOpacity = 0;

                    FaceLayout layout = mask.Tag as FaceLayout;
                    Debug.Assert(layout != null);

                    mask.MaskWidth = layout.Width;
                    mask.MaskHeight = layout.Height;
                    mask.Position = new System.Windows.Point(layout.CenterX, layout.CenterY);

                    mask.RenderTransformOrigin = new System.Windows.Point(0.5, 0.5);
                    var ct = mask.RenderTransform as CompositeTransform;
                    ct.ScaleX = 1;
                    ct.ScaleY = 1;
                    ct.Rotation = layout.Angle;

                    mask.Visibility = System.Windows.Visibility.Visible;
                    nonShownFaces.Remove(layout.Wrapper);
                }
            }

            foreach (var f in nonShownFaces)
            {
                var mask = CreateMask(f);
                this.maskCanvas.Children.Add(mask);
            }
        }

        /// <summary>
        /// Create a face mask based for the given face
        /// </summary>
        private MaskControl CreateMask(FaceWrapper wrapper)
        {
            FaceLayout l = new FaceLayout();
            l.Wrapper = wrapper;

            var f = wrapper.Face;
            Debug.Assert(f != null);

            // estimate the rotation angle
            l.Angle = GetFaceInplaneRotationAngle(f.Shape);
            l.CenterX = this.transform.XImageToView(f.Region.Center.X);
            l.CenterY = this.transform.YImageToView(f.Region.Center.Y);
            l.Width = f.Region.Width * this.transform.ScaleX * HorizontalScaleOfFaceRect;
            l.Height = f.Region.Height * this.transform.ScaleY * VerticalScaleOfFaceRect;

            // append uniform strech tranlation;
            if (transform.Strech == Stretch.Uniform)
            {
                l.CenterY += (this.imgGrid.Height - this.imgViewer.ActualHeight) / 2;
            }

            MaskControl mask = new MaskControl();
            mask.MaskWidth = l.Width;
            mask.MaskHeight = l.Height;
            mask.IsDragable = false;
            mask.IsSelectable = false;
            mask.Position = new System.Windows.Point(l.CenterX, l.CenterY);

            mask.RenderTransformOrigin = new System.Windows.Point(0.5, 0.5);
            var ct = mask.RenderTransform as CompositeTransform;
            ct.Rotation = l.Angle;

            mask.ButtonClicked += this.OnRemoveMask;
            mask.Tag = l;

            return mask;
        }

        /// <summary>
        /// Update mask layout when the orientation changes
        /// </summary>
        private void UpdateMaskLayout()
        {
            foreach (var element in this.maskCanvas.Children)
            {
                var mask = element as MaskControl;
                if (mask != null)
                {
                    FaceLayout l = mask.Tag as FaceLayout;
                    Debug.Assert(l != null);

                    FaceWrapper wrapper = l.Wrapper;
                    Debug.Assert(wrapper != null);

                    // Get the mask's translation manipulated by user
                    double tranlateX = mask.Position.X - l.CenterX;
                    double tranlateY = mask.Position.Y - l.CenterY;
                    double oldScaleX = l.Width / HorizontalScaleOfFaceRect / wrapper.Face.Region.Width;
                    double oldScaleY = l.Height / VerticalScaleOfFaceRect / wrapper.Face.Region.Height;

                    if (oldScaleX == 0 || oldScaleY == 0)
                    {
                        continue;
                    }

                    tranlateX *= (this.transform.ScaleX / oldScaleX);
                    tranlateY *= (this.transform.ScaleY / oldScaleY);

                    // Calculate mask's new position and size
                    l.CenterX = this.transform.XImageToView(wrapper.Face.Region.Center.X);
                    l.CenterY = this.transform.YImageToView(wrapper.Face.Region.Center.Y);
                    l.Width = wrapper.Face.Region.Width * this.transform.ScaleX * HorizontalScaleOfFaceRect;
                    l.Height = wrapper.Face.Region.Height * this.transform.ScaleY * VerticalScaleOfFaceRect;

                    // append uniform strech tranlation;
                    if (transform.Strech == Stretch.Uniform)
                    {
                        l.CenterY += (this.PhotoFrameGrid.ActualHeight - this.transform.SourceSize.Height * this.transform.ScaleY) / 2;
                        l.CenterX += (this.PhotoFrameGrid.ActualWidth - this.transform.SourceSize.Width * this.transform.ScaleX) / 2;
                    }

                    mask.MaskWidth = l.Width;
                    mask.MaskHeight = l.Height;

                    // Compute mask position.
                    mask.Position = new System.Windows.Point(l.CenterX + tranlateX, l.CenterY + tranlateY);
                }
            }
        }

        DateTime lastShake = DateTime.Now;
        static TimeSpan ShakeInterval = TimeSpan.FromSeconds(1);

        SoundEffect successEffect = null;
        void PlaySuccessSound()
        {
            if (successEffect == null)
            {
                Stream stream = TitleContainer.OpenStream("Assets/Sound/WindowsShutdown.wav");
                successEffect = SoundEffect.FromStream(stream);
                FrameworkDispatcher.Update();
            }

            successEffect.Play();
        }
        

    }
}
