﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Windows;
using System.Diagnostics;

namespace FaceParty
{
    public class ImageTransform
    {
        #region Private fields

        private System.Windows.Size sourceSize;
        private System.Windows.Size viewSize;
        private System.Windows.Media.Stretch stretch = System.Windows.Media.Stretch.None;

        #endregion
        #region Public methods

        /// <summary>
        /// Convert X coordinate in view space to image space
        /// </summary>
        /// <param name="x">X coordinate in view space</param>
        /// <returns>X coordinate in image space</returns>
        public double XViewToImage(double x)
        {
            Debug.Assert(this.ScaleX != 0);
            var imgX = (x - this.OriginX) / this.ScaleX;
            return imgX;
        }

        /// <summary>
        /// Convert Y coordinate in view space to image space
        /// </summary>
        /// <param name="y">Y coordinate in view space</param>
        /// <returns>Y coordinate in image space</returns>
        public double YViewToImage(double y)
        {
            Debug.Assert(this.ScaleY != 0);
            var imgY = (y - this.OriginY) / this.ScaleY;
            return imgY;
        }

        /// <summary>
        /// Convert X coordinate in image space to view space
        /// </summary>
        /// <param name="x">X coordinate in image space</param>
        /// <returns>X coordinate in view space</returns>
        public double XImageToView(double x)
        {
            var viewX = x * this.ScaleX + this.OriginX;
            return viewX;
        }

        /// <summary>
        /// Convert Y coordinate in image space to view space
        /// </summary>
        /// <param name="y">Y coordinate in image space</param>
        /// <returns>Y coordinate in view space</returns>
        public double YImageToView(double y)
        {
            var viewY = y * this.ScaleY + this.OriginY;
            return viewY;
        }

        #endregion
        #region Public properties

        public double ScaleX { get; protected set; }
        public double ScaleY { get; protected set; }
        public double OriginX { get; protected set; }
        public double OriginY { get; protected set; }

        public Size SourceSize
        {
            get { return this.sourceSize; }
            set 
            {
                if (this.sourceSize != value)
                {
                    this.sourceSize = value;
                    this.Measure();
                }
            }
        }

        public Size ViewSize
        {
            get { return this.viewSize; }
            set 
            {
                if (this.viewSize != value)
                {
                    this.viewSize = value;
                    this.Measure();
                }
            }
        }

        public System.Windows.Media.Stretch Strech
        {
            get { return this.stretch; }
            set
            {
                if (this.stretch != value)
                {
                    this.stretch = value;
                    this.Measure();
                }
            }
        }

        #endregion
        #region Private methods
        
        private void Measure()
        {
            if (this.sourceSize == System.Windows.Size.Empty)
                return;

            switch (this.Strech)
            {
                case System.Windows.Media.Stretch.None:
                    // Don't scale the image and the image is center-placed in the view region
                    this.ScaleX = this.ScaleY = 1;
                    this.OriginX = (this.ViewSize.Width - this.SourceSize.Width) / 2;
                    this.OriginY = (this.ViewSize.Height - this.SourceSize.Height) / 2;
                    break;

                case System.Windows.Media.Stretch.Fill:
                    // Scale the image to fill the entire view region
                    this.ScaleX = this.ViewSize.Width / this.SourceSize.Width;
                    this.ScaleY = this.ViewSize.Height / this.SourceSize.Height;
                    this.OriginX = this.OriginY = 0;
                    break;

                case System.Windows.Media.Stretch.Uniform:
                    // Scale the image uniformally to ensure one of the size fill the view region and no image region 
                    // will be cliped
                    {
                        this.ScaleX = this.ViewSize.Width / this.SourceSize.Width;
                        this.ScaleY = this.ViewSize.Height / this.SourceSize.Height;

                        double scale = System.Math.Min(this.ScaleX, this.ScaleY);
                        double renderWidth = scale * this.SourceSize.Width;
                        double renderHeight = scale * this.SourceSize.Height;
                        this.OriginX = (this.ViewSize.Width - renderWidth) / 2;
                        this.OriginY = (this.ViewSize.Height - renderHeight) / 2;

                        this.ScaleX = scale;
                        this.ScaleY = scale;
                    }

                    break;

                case System.Windows.Media.Stretch.UniformToFill:
                    // Scale the image uniformally to fill the entire region
                    {
                        this.ScaleX = this.ViewSize.Width / this.SourceSize.Width;
                        this.ScaleY = this.ViewSize.Height / this.SourceSize.Height;

                        double scale = System.Math.Max(this.ScaleX, this.ScaleY);
                        double renderWidth = scale * this.SourceSize.Width;
                        double renderHeight = scale * this.SourceSize.Height;
                        this.OriginX = (this.ViewSize.Width - renderWidth) / 2;
                        this.OriginY = 0;

                        this.ScaleX = scale;
                        this.ScaleY = scale;
                    }

                    break;
            }
        }

        #endregion
    }
}
